#!/bin/bash

#skript spustí sám sebe v pozadí a ohlídá spnění následujících podmínek:
#
#	1. PPP rozhraní existje
#	2. PPP rozhraní má přidělenu IPv6 adresu
#	3. LAN rozhraní dostalo IPv6 adresu
#
#skript je spuštěn v /etc/network/interfaces v post-up-fázi
#PPP rozhraní a aby proces nebrzdil, spustí sám sebe v pozadí

####################
#     DEFINICE     #
####################

SELF=/sbin/start-dhcp6.sh
SYSLOGTAG=DHCP6_STARTER #abychom výstupy skriptu v systémovém logu snadno našli, použijeme tag
START_DHCP6_CLIENT='/usr/sbin/service wide-dhcpv6-client start' #příkaz startující DHCP6 klienta
IFCONFIG=/sbin/ifconfig
IFACE_DEVID_PPP=ppp0 #virtuální WAN rozhraní aneb PPPoE spojení
IFACE_DEVID_LAN=enp1s0 #LAN rozhraní
TEST_PERIOD_IN_S=25 #pokud test podmínky selže, počká s další zkouškou daný počet vteřin
MAX_TRY_COUNT=11 #po vypotřebování tohoto počtu pokusů skončí s fatální chybou

#podmínky:
#(zde je nutno upozornit, že prefix, není-li pevný, se může měnit!)
EXPECTED_ADDRESS_PATTERN_PPP='inet6 2a00.*::b'
#mění-li se prefix, mění se i adresa! (mám pevný prefix a DHCP6 klient je nastaven, aby LAN rozhraní přiděloval stále stejnou IPv6 adresu)
EXPECTED_ADDRESS_LAN='2a00:1028:cc00:43b::b'

#zapíše do systémového logu data předaná rourou - používá se pro přesměrování STDOUT a STDERR příkazu do logu
log_piped () {
	read msg
	logger -t $SYSLOGTAG $msg
}

#zapíše danou zprávu do systémového logu
log () {
	echo $@|log_piped
}

#zapíše do logu chybové hlášení a skončí s kódem 255
fail () {
	log "FATAL: $@"
	exit 255
}

#zkontroluje stav PPP rozhraní a vrátí nulu, splňuje-li stav kriteria - tedy má-li toto přidělenu globální IPv6 adresu
check_ppp_interface () {
	log "cecking interface $IFACE_DEVID_PPP, whether ifconfig put out any line matching '$EXPECTED_ADDRESS_PATTERN_PPP'"
	$IFCONFIG $IFACE_DEVID_PPP|grep "$EXPECTED_ADDRESS_PATTERN_PPP" > /dev/null
	return $?
}

#provede obdobný test nad LAN rozhraním - tedy opět zjistí, je-li toto nakonfigurováno včetně přidělené globální IPv6 adresy
check_intranet_interface () {
	log "cecking interface $IFACE_DEVID_LAN, whether it has got the expected address $EXPECTED_ADDRESS_LAN"
	$IFCONFIG $IFACE_DEVID_LAN|grep "inet6 $EXPECTED_ADDRESS_LAN" > /dev/null
	return $?
}

#čeká na splnění podmínky, přijímá dva parametry - jméno testovací funkce (viz výše) a jméno testovaného zařízení pro logování
wait_for_condition () {
	function=$1
	dev=$2

	tries_cntdn=$MAX_TRY_COUNT
	$function
	fail=$?
	#wait until condition is not fulfilled
	while [ $fail -ne 0 ]; do
		((tries_cntdn--))
		log "device $dev is not ready yet, tries left: $tries_cntdn"
		if [ $tries_cntdn -eq 0 ]; then
			fail "the device $dev is still down or hasn't got any IPv6 address yet"
		fi
		log "going to sleep for $TEST_PERIOD_IN_S sec"
		sleep $TEST_PERIOD_IN_S
		$function
		fail=$?
	done
}

#provede příkaz; STDOUT a STDERR jsou přesměrovány do logu a pokud příkaz selže, skript se ukončí s fatální chybou
eval_command () {
	log "CMD: '$@'"
	eval "$@" 2>&1|log_piped
	if [ ${PIPESTATUS[0]} -ne 0 ]; then
		fail "command '$@' failed"
	fi
}

#spustí sám sebe v pozadí, pokud není zadán žádný parametr na příkazové řádce
if [ -z "$1" ] ; then
	log 'going to execute my self in background'
	$SELF go &
	exit 0
fi

#####################
# SKRIPT ZAČÍNÁ ZDE #
#####################

#čekáme na PPP rozhraní
wait_for_condition check_ppp_interface $IFACE_DEVID_PPP
log 'first condition fulfilled, going to start DHCP6 client'

#poté spustíme DHCP6 klienta
eval_command $START_DHCP6_CLIENT

#ještě zkontrolujeme, zda vše proběhlo správně a LAN rozhraní dostane přidělenu očekávanou IPv6 adresu
wait_for_condition check_intranet_interface $IFACE_DEVID_LAN

#(tady se bude později spouštět ještě DHCP6 server, ale pro tento návod je toto irelevantní)

log '[ ALL DONE ]'

exit 0

#_______________________________
#TK, barbucha.eu (c) 2o2o-11-22
